﻿
BrokerItemlist = LibStub("AceAddon-3.0"):NewAddon("Broker_Itemlist", "AceBucket-3.0")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local QTC = LibStub('LibQTip-1.0')
local C = LibStub("AceConfigDialog-3.0")

local mod = BrokerItemlist
local db
local PLAYER_LEVEL = UnitLevel("player")

local L = LibStub("AceLocale-3.0"):NewLocale("Broker_Itemlist", "enUS", true)
	
if L then
	L["Items"] = true
	L["Items you wish to monitor."] = true
	L["Settings"] = true
	L["Show in text"] = true
	L["Items you wish to show in text field."] = true
	L["Dev Mode"] = true
	L["Icon size"] = true
	L["Add item"] = true
	L["Add new item to track.  Drag the item to this input box."] = true
	L["Delete item"] = true
	L["List of current items.  Click to delete item."] = true
end

local L = LibStub("AceLocale-3.0"):GetLocale("Broker_Itemlist")

local playerClass
local ItemInfo = {}
local sorted = {}

local ClassIcons = { 	DRUID = "Interface\\Icons\\INV_Misc_Branch_01", MAGE = "Interface\\Icons\\INV_Misc_Dust_01", PALADIN = "Interface\\Icons\\INV_Stone_WeightStone_05",
			PRIEST = "Interface\\Icons\\INV_Misc_Candle_03", ROGUE = "Interface\\Icons\\Trade_BrewPoison", SHAMAN = "Interface\\Icons\\INV_Jewelry_Talisman_06",
			WARLOCK = "Interface\\Icons\\INV_Misc_Gem_Amethyst_02", DEATHKNIGHT = "Interface\\Icons\\INV_Misc_Dust_02", HUNTER = "Interface\\Icons\\INV_Ammo_Arrow_02"
}

local ReagentDefaults = { 	DRUID = {17021, 17026, 22148, 17034, 17035, 17036, 17037, 17038, 22147, 44614, 44605},
				MAGE = {17020, 17031, 17032, 17056}, PALADIN = {17033, 21177}, PRIEST = {17028, 17029, 44615, 17056},
				ROGUE = {7676, 6947, 2892, 3775, 5237, 10918, 21835}, SHAMAN = {17030, 17057, 17058},
				WARLOCK = {6265, 5509, 16893, 41191, 41169, 5565, 16583}, DEATHKNIGHT = {37201},
				HUNTER = {}, WARRIOR = {}
}

local SpecialDefaults = {	40752, 	--emblem of heroism
				40753, 	--emblem of valor
				29434,	--badge of justice
				44711,	--argent crusade badge
				44713,	--ebon blade badge
				43950,	--kirin tor badge
				44710,	--wyrmrest badge
				41596,	--dalaran JC token
}

local QualityColor = {"|c009d9d9d", "|c00ffffff", "|c001eff00", "|c000070dd", "|c00a335ee", "|c00ff8000", "|c00e6cc80"}

local defaults = {
	char = {
		devmode = nil,
		firstrun = true,
		iconsize = 28,
		tracking = {},
		text = {}
	}
}

local options = {
	name = "Broker Itemlist",
	handler = mod,
	type = "group",
	args = { 
		items = {
		 	type = "multiselect",
		 	name = L["Items"],
		 	desc = L["Items you wish to monitor."],
		 	get = function(info, v) 
		 			return mod:ItemExists(v) 
		 		end,
		 	set = function(info, name) 
		 			mod:ToggleItem(name)
		 		end,
		 	values = "GetItems", 
			order = 5
		},
		text = {
		 	type = "multiselect",
		 	name = L["Show in text"],
		 	desc = L["Items you wish to show in text field."],
		 	get = function(info, v) 
		 			return mod:ItemOnText(v) 
		 		end,
		 	set = function(info, name) 
		 			mod:ToggleTextItem(name)
		 		end,
		 	values = "GetItems", 
			order = 8
		},
		head = {
			type = "header",
			name = L["Settings"],
			order = 10
		},
		additem = {
			type = "input",
			name = L["Add item"],
			desc = L["Add new item to track.  Drag the item to this input box."],
			set = 	function(info, v) mod:AddNewItem(v) end,
			order = 12
		},
		delitem = {
			type = "select",
			name = L["Delete item"],
			desc = L["List of current items.  Click to delete item."],
			set = function(info, v) mod:RemoveItem(v) end,
			values = "GetItems",
			order = 15
		},
		iconsize = {
			type = "range",
			name = L["Icon size"],
			get = function() return mod.db.char.iconsize end,
			set = 	function(info, v) mod.db.char.iconsize = v mod:UpdateBrokerText() end,
			min = 8, 
			max = 40, 
			step = 1,
			order = 20
		},
		dev = {
			type = "toggle",
			name = L["Dev Mode"],
			get = function() return mod.db.char.devmode end,
			set = function(info, v) mod.db.char.devmode = v end,
			order = 50
		},
	}
}
		

function mod:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BrokerItemlistDB", defaults)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Broker_Itemlist", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Broker_Itemlist", "Broker Itemlist")
	
	_, playerClass = UnitClass("player")
	if mod.db.char.firstrun then
		for _, item in pairs(SpecialDefaults) do
			mod.db.char.tracking[item] = true
		end
		for _, item in pairs(ReagentDefaults[playerClass]) do
			mod.db.char.tracking[item] = true
		end
		mod.db.char.firstrun = false
	end
	self:CreateLDBObject()
end

function mod:OnEnable()
	db = mod.db.char
	
	for item in pairs(db.tracking) do
		self:AddItem(item)
	end
	self:RegisterBucketEvent("BAG_UPDATE", 1)	
end

function mod:OnDisable()
	
end

function mod:debug(message)
	if db.devmode then
		DEFAULT_CHAT_FRAME:AddMessage(message)
	end
end

function mod:ItemExists(item)
	for itemid, data in pairs(ItemInfo) do
		if data.name == item then
			if db.tracking[itemid] then return true, itemid 
			else return false, itemid end
		end
	end 
	return nil
end

function mod:ToggleItem(name)
	local result, itemid = mod:ItemExists(name) 
	if itemid then 
		db.tracking[itemid] = not db.tracking[itemid]
	end
	self:UpdateBrokerText()
end

function mod:ItemOnText(itemname)
	for itemid, data in pairs(ItemInfo) do
		if data.name == itemname then
			if db.text[itemid] then return true, itemid 
			else return false, itemid end
		end
	end 
	return nil
end

function mod:ToggleTextItem(name)
	local result, itemid = mod:ItemOnText(name) 
	if itemid then 
		db.text[itemid] = not db.text[itemid]
	end
	self:UpdateBrokerText()
end

function mod:GetItems()
	local list = {}
	for item in pairs(db.tracking) do
		if ItemInfo[item] then
			list[ItemInfo[item].name] = ItemInfo[item].color .. ItemInfo[item].name .. "|r"
		end
	end
	return list
end

function mod:AddNewItem(itemlink)
	if not itemlink or itemlink == "" then return end
	local _,_, item = string.find(itemlink, "item:(%d+):")
	item = tonumber(item)
	if not db.tracking[item] then
		db.tracking[item] = true
		self:AddItem(item)
		self:BAG_UPDATE()
	end 
end

function mod:RemoveItem(itemname)
	if not itemname then return end
	local result, item = self:ItemExists(itemname)
	if result and item then
		db.tracking[item] = nil
		if db.text[item] then db.text[item] = nil end
		self:UpdateBrokerText()
	end
end

function mod:CreateLDBObject()
	mod.obj = ldb:NewDataObject("Broker_Itemlist", {
		type = "data source",
		icon = ClassIcons[playerClass],
		label = "Itemlist",
		OnClick = function(frame, button)
				if button == "RightButton" then
					if not C.OpenFrames["Broker_Itemlist"] then C:Open("Broker_Itemlist") 
					else C:Close("Broker_Itemlist") end
				else 
				
				end
			end,
		OnEnter = function(frame)
				local tooltip = QTC:Acquire("Broker_ItemlistTooltip", 3, "LEFT", "LEFT", "RIGHT")
				tooltip:Clear()
				tooltip:SmartAnchorTo(frame)
				tooltip:SetScript("OnLeave", mod.HideTooltip)
				mod.tooltip = tooltip
				mod:DisplayTooltip()
			end,
		OnLeave = function()
				mod:HideTooltip()
			end,
	})

end

function mod:HideTooltip()
	mod.tooltip:Hide()
	QTC:Release(mod.tooltip)
	mod.tooltip = nil
end

function mod:PrepareTooltipLook(frame)
	frame:SetBackdrop(GameTooltip:GetBackdrop())
	frame:SetBackdropBorderColor(GameTooltip:GetBackdropBorderColor())
	frame:SetBackdropColor(GameTooltip:GetBackdropColor())
	frame:SetScale(GameTooltip:GetScale())
end

function mod:UpdateBrokerText()
	local icons
	local text = ""
	for itemid, show in pairs(db.text) do
		if show and ItemInfo[itemid] then
			local data = ItemInfo[itemid]
			text = text .. "|T" .. data.icon .. ":" .. db.iconsize .. "|t"
			local color
			if data.count < data.max then color = "|c00FFFF00"
			else color = "|c0000FF00" end 
			text = text .. color .. data.count .. "|r"
			icons = true
		end
	end
	if icons then
		mod.obj.text = text
	else 
		mod.obj.text = ""
	end
end

function mod:DisplayTooltip()
	self:PrepareTooltipLook(mod.tooltip)
	
	sorted = {}
	for itemid, show in pairs(db.tracking) do
		if show and ItemInfo[itemid] then
			local data = ItemInfo[itemid]
			table.insert(sorted, {name = data.name, icon = data.icon, count = data.count, max = data.max, color = data.color, quality = data.quality})
		end
	end
	table.sort(sorted, function(a,b) 
				if a.quality == b.quality then
					return a.name < b.name 
				else
					return a.quality > b.quality
				end
	end)
	for _, data in ipairs(sorted) do
		local icontext = "|T" .. data.icon .. ":" .. db.iconsize .. "|t"
		local nametext = data.color .. data.name .. "|r"
		local color
		if data.count < data.max then color = "|c00FFFF00"
		else color = "|c0000FF00" end 
		local counttext = color .. data.count .. "|r"
		mod.tooltip:AddLine(icontext, nametext, counttext)
	end
			
	mod.tooltip:Show()
end

function mod:BAG_UPDATE()
	for item in pairs(db.tracking) do
		self:AddItem(item)
		if not ItemInfo[item] then return end
		ItemInfo[item].count = 0
		if ItemInfo[item].type == "Money" or ItemInfo[item].type == "Money(OBSOLETE)" then self:UpdateSpecialItem(item) end
	end
	for bag = 4, 0, -1 do
		local size = GetContainerNumSlots(bag)
		if (size > 0) then
			for slot = 1, size, 1 do
				local _, itemCount = GetContainerItemInfo(bag, slot)
				if itemCount then
					itemName, itemLink, _, _, _, _, _, _, _, itemTexture = GetItemInfo(GetContainerItemLink(bag, slot))
					if itemName then
						local _,_, itemId = string.find(itemLink, "item:(%d+):")
						itemId = tonumber(itemId)
						if db.tracking[itemId] then
							ItemInfo[itemId].count = ItemInfo[itemId].count + itemCount
						else
							for trackitem in pairs(db.tracking) do
								if ItemInfo[trackitem] then
									if string.find(itemName, ItemInfo[trackitem].name) then
										ItemInfo[trackitem].count = ItemInfo[trackitem].count + itemCount
										break
									end
								end
							end
						end
					end
				end
			end
		end
	end
	self:UpdateBrokerText()
end

function mod:AddItem(item)
	if not ItemInfo[item] then
		itemName, _, itemRarity, _, _, itemType, _, itemStackCount, _, itemTexture = GetItemInfo(item)
		if not itemName then 
			self:ScanTooltip(item)
			return
		end
		ItemInfo[item] = {name = itemName, icon = itemTexture, count = 0, max = itemStackCount, color = QualityColor[itemRarity + 1], quality = itemRarity, type = itemType}
	end
end

function mod:UpdateSpecialItem(item)
	if not ItemInfo[item] then return end
	local name = ItemInfo[item].name
	for i=1,GetCurrencyListSize() do
      		local tokenname, _, _, _, _, qty = GetCurrencyListInfo(i)
      		if tokenname == name then 
    			ItemInfo[item].count = qty 
    			break
    		end
    	end
end

function mod:ScanTooltip(item)
	self:debug("Start scanning .."..item)
	if not self.Scanner then 
		self.Scanner = CreateFrame("GameTooltip", "Itemlist_ScannerTooltip", nil, "GameTooltipTemplate")
		self.Scanner:SetOwner(UIParent, "ANCHOR_NONE")
	end
	self.Scanner:SetHyperlink('item:'..item)
	self.Scanner.id = item
	self.Scanner:SetScript('OnTooltipSetItem', function() return mod:OnTooltipSetItem() end)
end

function mod:OnTooltipSetItem()
	mod:debug("Returned itemlookup")
	if GetItemInfo(mod.Scanner.id) then
		mod.Scanner:SetScript('OnTooltipSetItem', nil)
		mod:AddItem(mod.Scanner.id)
	end
end
